#!/usr/bin/env python

## Filename:      analyzer.py
## Description:   file magic + decompression
## Author:        David Dumas <ddumas@math.uic.edu>
## Modified at:   Fri Jul  9 21:26:43 2010
##                
## Copyright (C) 2005, 2010  David Dumas
##                
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
##
## See the file COPYING for details.

"""
Analyze a file (or URL to be downloaded) containing a document of some type,
possibly compressed, possibly having erroneous or missing extensions (.tar,
.gz, etc).  The file is decompressed, renamed if appropriate, and an attempt
is made to extract text for display as a preview.

Right now only PS and PDF are supported, with GZIP or BZIP2 compression.
"""

import sys
import os, os.path
import warnings
from urlparse import urlparse
from shutil import copyfileobj, move
import string
import re
import logging
import md5

# "extractor" module is part of dfiler
import extractor

try:
    import urlgrabber as url_module
    _URL_MODULE = 'urlgrabber'
except ImportError:
    import urllib2 as url_module
    _URL_MODULE = 'urllib2'

warnings.filterwarnings("ignore", "tempnam", RuntimeWarning, __name__)

_TEMPDIR = '/tmp'

class Analyzer(object):
    url=None
    fn=None
    abspath=None
    dirname=None
    local=False
    magic=None
    recognized_type=None
    mtype = None
    text=None
    printable_text=None
    progress = None
    logger = None
    def __init__(self, url=None, progress=None):
        self.logger = logging.getLogger('Analyzer')
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug('__init__')
        if progress:
            self.progress = progress
        if url:
            self.get(url)
        
    def filemagic(self,fn=None):
        if not fn:
            fn = self.fn
        p = os.popen('file -b "%s"' % fn)
        l = p.readline()[:-1]
        p.close()
        return l

    def tempnam(self):
        return  os.tempnam(_TEMPDIR,'dfiler')

    def get(self,url,decomp=True,remove_extensions=True,text=True,progress=None):
        self.logger.debug('get')
        self.url = url
        if progress:
            self.progress = progress
        url_parts = urlparse(url)
        if url_parts[0] in ['', 'file']:
            # looks like a local file, try to open it
            self.fn = url_parts[2]
            if not os.path.exists(self.fn):
                sys.exit('Error: file "%s" does not exist' % self.fn)
            if not os.path.isfile(self.fn):
                sys.exit('Error: "%s" is not a regular file' % self.fn)
            if os.path.islink(self.fn):
                sys.exit('Error: cannot operate on symbolic link "%s"' % self.fn)
            try:
                resource = file(self.fn)
            except IOError:
                sys.exit('Error: unable to open file "%s"' % self.fn)
            self.local = True
            self.abspath = os.path.abspath(self.fn)
            self.dirname = os.path.dirname(self.abspath)
        else:
            # url points to remote resource, try to open it
            try:
                self.logger.debug('url_module.urlopen("%s")' % url)
                if _URL_MODULE == 'urlgrabber':
                    remote_resource = url_module.urlopen(url,user_agent="Mozilla/5.0",progress_obj=self.progress)
                else:
                    remote_resource = url_module.urlopen(url)
                self.logger.debug('url_module.urlopen completed')
            except:
                sys.exit('Error: unable to open URL "%s"' % url)
            self.local = False
            # open a temp file to hold the resource
            self.fn = self.tempnam()
            try:
                resource = file(self.fn,"wb")
            except IOError:
                sys.exit('Error: unable to open temporary file "%s"')
            self.logger.info('downloading...')
            copyfileobj(remote_resource,resource)
            self.logger.info('done downloading; wrote "%s"' % self.fn)
            remote_resource.close()
            resource.close()
        self.magic = self.filemagic()
        if decomp:
            self.decomp()
        if self.local and remove_extensions:
            self.remove_extensions()
        if text:
            self.get_text()

    def decomp(self):
        # if the resource is a compressed file, uncompress it
        m = self.magic
        decompressed = False
        if m.split()[0] == 'gzip':
            self.logger.info('decompressing...')
            if self.local:
                if self.fn[-3:] == '.gz':
                    os.system('gzip -d "%s"' % self.fn)
                    decompressed = True
                    self.fn = self.fn[:-3]
                elif self.fn[-4:] == '.tgz':
                    os.system('gzip -d "%s"' % self.fn)
                    decompressed = True
                    self.fn = self.fn[:-4]
                else:
                    r = os.system('gzip -c -d < "%s" > "%s.1"' % (self.fn,self.fn))
                    if r==0 and os.path.exists(self.fn+'.1'):
                        move(self.fn+'.1',self.fn)
                        decompressed = True
                    else:
                        sys.exit('Error: unable to decompress gzip file "%s"' % self.fn)
            else:
                fn2 = tempnam()
                r = os.system('gzip -c -d < "%s" > "%s"' % (self.fn,fn2))
                if r==0 and os.path.exists(fn2):
                    os.unlink(self.fn)
                    decompressed = True
                self.fn = fn2
            self.logger.info('done decompressing; wrote to "%s"' % self.fn)

        if m.split()[0] == 'bzip2':
            self.state = 'decompress'
            self.logger.info('decompressing...')
            if self.local:
                if self.fn[-3:] == '.bz':
                    os.system('bzip2 -d "%s"' % self.fn)
                    decompressed = True
                    self.fn = self.fn[:-3]
                elif self.fn[-4:] == '.bz2':
                    os.system('bzip2 -d "%s"' % self.fn)
                    decompressed = True
                    self.fn = self.fn[:-4]
                else:
                    r = os.system('bzip2 -c -d < "%s" > "%s.1"' % (self.fn,self.fn))
                    if r==0 and os.path.exists(self.fn+'.1'):
                        move(self.fn+'.1',self.fn)
                        decompressed = True
                    else:
                        sys.exit('Error: unable to decompress bzip file "%s"' % self.fn)
            else:
                fn2 = tempnam()
                r = os.system('bzip2 -c -d < "%s" > "%s"' % (self.fn,fn2))
                if r==0 and os.path.exists(fn2):
                    os.unlink(self.fn)
                    decompressed = True
                self.fn = fn2
            self.logger.info('done decompressing; wrote to "%s"' % self.fn)
        if decompressed:
            self.magic = self.filemagic()

    def remove_extensions(self):
        fns = self.fn.split('.')
        if (len(fns) > 1) and (fns[-1] in ['gz','tgz','bz2','bz','tbz2','tbz']) and not ('compressed' in self.magic):
            fn2 = string.join(fns[:-1],'.')
            if os.path.exists(fn2):
                sys.exit('Error: will not overwrite existing file "%s"' % fn2)
            move(self.fn,fn2)
            self.fn = fn2
            self.logger.info('removed erroneous compression suffix "%s" (filename now "%s")' % (fns[-1],self.fn))

    def get_text(self):
        self.logger.info('trying to get text from file with magic "%s"' % self.magic)
        if 'pdf' in self.magic.lower():
            self.recognized_type = 'pdf'
            self.mtype = 'application/pdf'
        elif 'postscript' in self.magic.lower():
            self.recognized_type = 'ps'
            self.mtype = 'application/postscript'
        self.text = extractor.TextExtractor(self.fn,self.recognized_type)
        self.printable_text = self.clip(self.text)

    def clip(self,text=None):
        if not text:
            text = self.text
        return [x[:80] for x in text[:10]]


if __name__ == '__main__':
    logging.basicConfig()
    if len(sys.argv) < 2:
        sys.exit('Error: empty URL')

    url = sys.argv[1]
    analyzer = Analyzer(url)
    print 'url:\t',analyzer.url
    print 'fn:\t',analyzer.fn
    print 'type:\t',analyzer.recognized_type
    print 'text:'
    for l in analyzer.printable_text:
        print l

