#!/usr/bin/env python

## Filename:      dfiler.py
## Description:   dfiler v0.0.5
## Author:        David Dumas <ddumas@math.brown.edu>
## Modified at:   Tue Jul 27 11:09:00 2010
##                
## Copyright (C) 2005-2010 David Dumas
##                
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## See the file COPYING for details.

"""
dfiler: download, decompress, display, and store a preprint.
"""

_DFILER_VERSION = '0.0.5'

# early arg check, because wxpython takes lots of memory/time to load

import sys
if __name__ == '__main__':
    if len(sys.argv) < 2:
        sys.exit('usage: %s [URL | FILENAME]\n' % sys.argv[0]) 

# WANT wxpython 2.5+, ALLOW >= 2.4
try:
    import wxversion
    WXVERSIONS = ['2.8', '2.7', '2.6','2.5']
    WXMIN = '2'
    selected = False
    for x in WXVERSIONS:
        if wxversion.checkInstalled(x):
            wxversion.select(x)
            selected = True
            break
    if not selected:
        sys.stderr.write('Warning: dfiler was developed for wxPython 2.5-2.8; deteced installed versions: '+str(wxversion.getInstalled())+'\n')
        wxversion.ensureMinimal(WXMIN)
    import wx
except ImportError:
    import wx
    if wx.VERSION[:2] < (2,5) or wx.VERSION[:2] > (2.8):
        sys.stderr.write('Warning: dfiler was developed for wxPython 2.5-2.8; deteced installed versions: '+str(wxversion.getInstalled())+'\n')
    
import os, os.path, shutil
import warnings
from urlparse import urlparse
import string
import re
import logging
import threading
import mailcap
import md5

# "analyzer" module is part of dfiler
import analyzer

class _settings:
    HOMEDIR = os.getenv('HOME','/home/ddumas')
    BASEDIR = HOMEDIR + '/preprints'
    TEXT_PREVIEW_FONT_NAME = 'terminus'
#    TEXT_PREVIEW_FONT_NAME = 'monospace'
    VIEW_ON_EXIT = True
    USE_UNICODE = True  # If available

# Unicode availability check
def wxHasUnicode():
    try:
        return('unicode' in wx.PlatformInfo)
    except AttributeError:
        if 'USE_UNICODE' in dir(wx):
            return wx.USE_UNICODE
        elif wxUSE_UNICODE in dir(wx):
            return wx.wxUSE_UNICODE
        else:
            return FALSE

# wxwidgets may or may not (!) know about unicode
# If not, this function scrubs cuts a unicode string down
# to something 7-bit clean.
# Also: force scrub if _settings.USE_UNICODE is False
def wxscrub(s):
    if _settings.USE_UNICODE:
        return s
    else:
        if type(s) == unicode:
            return s.encode('ascii','ignore')
               
# GUI declarations
STATECHG = wx.NewEventType()
FILEINFO = wx.NewEventType()
NEWSELECTION = wx.NewEventType()
GAUGEUPDATE = wx.NewEventType()

def EVT_STATECHG(win, func):
    win.Connect(-1, -1, STATECHG, func)

def EVT_FILEINFO(win, func):
    win.Connect(-1, -1, FILEINFO, func)

def EVT_NEWSELECTION(win, func):
    win.Connect(-1, -1, NEWSELECTION, func)

def EVT_GAUGEUPDATE(win, func):
    win.Connect(-1, -1, GAUGEUPDATE, func)

class StateChangeEvent(wx.PyCommandEvent): 
    eventType = STATECHG
    newstate = None
    def __init__(self, window, newstate=None): 
        wx.PyCommandEvent.__init__(self, self.eventType, window.GetId())
        self.newstate = newstate
    def Clone( self ): 
        self.__class__( self.GetId() ) 

class FileInfoEvent(wx.PyCommandEvent): 
    eventType = FILEINFO
    data = None
    def __init__(self, window, data): 
        wx.PyCommandEvent.__init__(self, self.eventType, window.GetId())
        self.data = data
    def Clone( self ): 
        self.__class__( self.GetId() ) 

class NewSelectionEvent(wx.PyCommandEvent): 
    eventType = NEWSELECTION
    selection = ''
    pos = (-1,-1)
    def __init__(self, window, s, pos): 
        wx.PyCommandEvent.__init__(self, self.eventType, window.GetId())
        self.selection = s
        self.pos = pos
    def Clone( self ): 
        self.__class__( self.GetId() ) 
    def GetPosition(self):
        return self.pos
    def GetSelection(self):
        return self.selection

class GaugeUpdateEvent(wx.PyCommandEvent): 
    eventType = GAUGEUPDATE
    newvalue = 0
    newrange = None
    def __init__(self, window, newvalue = 0, newrange = None): 
        wx.PyCommandEvent.__init__(self, self.eventType, window.GetId())
        self.newvalue = newvalue
        self.newrange = newrange
    def Clone( self ):
        self.__class__( self.GetId() ) 

class AnalyzerProgress(object):
    eventdest = None
    length = None
    cur = 0
    logger = None
    def __init__(self, eventdest):
        self.eventdest = eventdest
        self.logger = logging.getLogger('AnalyzerProgress')
        self.logger.setLevel(logging.INFO)
        self.logger.debug('__init__')
    def send_event(self):
        self.logger.debug('send_event')
        if self.length:
            self.eventdest.GetEventHandler().AddPendingEvent(GaugeUpdateEvent(self.eventdest,self.cur,self.length))
        else:
            self.eventdest.GetEventHandler().AddPendingEvent(GaugeUpdateEvent(self.eventdest,self.cur,1))
    def start(self,filename,url,basename,length):
        self.logger.debug('start')
        if length:
            self.logger.info('length=%d' % length)
        else:
            self.logger.info('length=unknown')
        self.length = length
        self.cur = 0
        self.send_event()
        self.logger.debug('~start')
    def update(self,read):
        self.logger.debug('update')
        if self.length:
            self.cur = read
        self.send_event()
        self.logger.debug('~update')
    def end(self,read):
        self.logger.debug('end')
        if not self.length:
            self.cur = 1
        self.send_event()

class AnalyzerThread(threading.Thread):
    mutex = threading.Lock()
    logger = logging.getLogger('AnalyzerThread')
    state = 'init'
    new = False
    d = { 'url':None, 'text':None, 'filename':None, 'ptext':None,
          'rtype':None, 'local':False, 'mtype':None, 'magic':None,
          'md5':None, 'size':None, 'mtime':None, 'abspath':None, 'dirname':None}
    a = None
    def __init__(self,owner,url):
        threading.Thread.__init__(self)
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug('__init__')
        self.mutex.acquire()
        self.d['url'] = url
        self.owner = owner
        self.progress = AnalyzerProgress(owner)
        self.mutex.release()

    def update_analyzer_data(self):
        self.d['filename'] = self.a.fn
        self.d['rtype'] = self.a.recognized_type
        self.d['text'] = self.a.text
        self.d['ptext'] = self.a.printable_text
        self.d['local'] = self.a.local
        self.d['magic'] = self.a.magic
        self.d['mtype'] = self.a.mtype
        self.d['dirname'] = self.a.dirname
        self.d['abspath'] = self.a.abspath

    def run(self):
        self.logger.debug('run')
        self.mutex.acquire()
        self.a = analyzer.Analyzer()
        self.state = 'download'
        self.owner.GetEventHandler().AddPendingEvent(StateChangeEvent(self.owner,self.state))
        self.mutex.release()
        self.logger.info('get(%s)' % self.d['url'])
        self.a.get(self.d['url'],decomp=False,remove_extensions=False,text=False,progress=self.progress)

        self.mutex.acquire()
        self.update_analyzer_data()
        self.state = 'process'
        self.owner.GetEventHandler().AddPendingEvent(StateChangeEvent(self.owner,self.state))
        self.mutex.release()

        self.logger.info('decompress')
        self.a.decomp()
        
        self.mutex.acquire()
        self.update_analyzer_data()
        self.owner.GetEventHandler().AddPendingEvent(StateChangeEvent(self.owner,self.state))
        self.mutex.release()

        if self.d['local']:
            self.logger.info('remove_extensions')
            self.a.remove_extensions()
            self.mutex.acquire()
            self.update_analyzer_data()
            self.mutex.release()

        self.logger.info('get_text')
        self.a.get_text()
        self.mutex.acquire()
        self.update_analyzer_data()
        self.get_file_data()
        self.mutex.release()

        self.mutex.acquire()
        self.state = 'done'
        self.owner.GetEventHandler().AddPendingEvent(StateChangeEvent(self.owner,self.state))
        self.mutex.release()

        self.mutex.acquire()
        self.owner.GetEventHandler().AddPendingEvent(FileInfoEvent(self.owner,self.d))
        self.mutex.release()

    def file_md5(self,f):
        m = md5.new()
        while True:
            s = f.read(65536)
            if not s:
                break
            m.update(s)
        return m.hexdigest()

    def get_file_data(self,fn=None):
        if not fn:
            fn = self.d['filename']
        st = os.stat(fn)
        self.d['size'] = st.st_size
        self.d['mtime'] = st.st_mtime
        f = file(fn)
        self.d['md5'] = self.file_md5(f)


class TextCtrlWithSelectionPopup(wx.TextCtrl):
    parent = None
    selection = ''
    def __init__(self,parent,popuphdlr,id,value="",pos=wx.DefaultPosition,size=wx.DefaultSize,style=0,validator=wx.DefaultValidator,name="TextCtrlWithPopup"):
        wx.TextCtrl.__init__(self,parent,id,value,pos,size,style,validator,name)
        self.hdlr = popuphdlr
        wx.EVT_LEFT_UP(self,self.OnLeftUp)

    def OnLeftUp(self,evt):
        selection=self.GetStringSelection()
        if len(selection) > 0:
            self.AddPendingEvent(NewSelectionEvent(self,selection,evt.GetPositionTuple()))
        evt.Skip()

class SelectionData(dict):
    def __init__(self,s,extension='',basedir=_settings.BASEDIR):
        if extension:
            self.extension = extension
        else:
            self.extension = ''
        self['oneline'] = re.sub('(\s|\W)+',' ',s.encode('ascii','replace')).strip()
        self['short'] = self['oneline'][:15]
        if len(self['oneline']) > 15:
            self['short'] = self['short'] + '...'
        self['words'] = self['oneline'].split(' ')
        self['word'] = self['words'][0]
        self['dirname'] = self['word'].lower()
        self['path'] = basedir + self['dirname']
        self['fn1'] = self.fncap(self['word'])+self.extension
        self['fn2'] = self.fncap(self['oneline'][:48])+self.extension

    def fncap(self,s):
        if s == s.upper():
            return string.capwords(s.lower())
        else:
            return s

class FileOpener(object):
    """Given a filename and a mimetype, find a viewer and launch on request"""
    # These hard-wired defaults are only used if mailcap mime data is not found
    opencmds = { 'application/pdf':'xpdf',
                 'application/postscript':'gv' }
    def __init__(self,fn=None,mime=None,mailcaps=None,usemailcap=True):
        self.logger = logging.getLogger('FileOpener')
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug('__init__')
        if usemailcap:
            if mailcaps:
                self.caps = mailcaps
            else:
                self.caps = mailcap.getcaps()
        else:
            self.caps = None
        self.mtype = mime
        self.fn = fn
    def IsKnown(self):
        if not self.mtype:
            return False
        if self.caps:
            return (mailcap.findmatch(self.caps,self.mtype) <> (None,None))
        else:
            return (self.mtype in self.opencmds.keys()) 
    def Launch(self):
        if (not self.fn) or (not self.mtype):
            self.logger.warning('Requested launch without filename and mimetype!')
            return 0
        if not self.IsKnown():
            self.logger.debug('Cannot launch unknown MIME type %s' % self.mtype)
            return 0
        if self.caps:
            mc = mailcap.findmatch(self.caps,self.mtype)
            if mc <> (None,None):
                self.logger.debug('Launching ',mc[0])
                p = mc[0].split(' ')  # fixme!
                pid = os.spawnlp(os.P_NOWAIT,p[0],p[0],self.fn)
                self.logger.debug('Viewer PID=%d' % pid)
                return pid
            else:
                self.logger.debug('MIME type %s not found in mailcap database.' % self.mtype)
        # fallback in case mailcap was enabled but failed
        if self.mtype in self.opencmds.keys():
            self.logger.debug('Launching ',self.opencmds[self.mtype])
            pid = os.spawnlp(os.P_NOWAIT,self.opencmds[self.mtype],self.opencmds[self.mtype],self.fn)
            self.logger.debug('Viewer PID=%d' % pid)
            return pid
        else:
            self.logger.debug('MIME type %s not in default viewer database.' % self.mtype)
            return 0

REPLACE_PATH_ID = wx.NewId()
REPLACE_FILENAME1_ID = wx.NewId()
REPLACE_FILENAME2_ID = wx.NewId()

class MyFrame(wx.Frame):
    state = 'init'
    logger = logging.getLogger('MyFrame')
    selection_data = {}
    file_data = {}
    base_path = _settings.BASEDIR + '/'
    mtm = None
    def __init__(self, parent, title, url=None):
        wx.Frame.__init__(self, parent, -1, title,(-1,-1),(700,550))   # Initial size set here
        self.SetSizeHints(350,300,-1,-1)  # minimum size set here
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug('__init__')

        self.state = 'init'
        self.url = url
        
        menuBar = wx.MenuBar()
        menu = wx.Menu()
        menu.Append(101, "E&xit\tAlt-X", "Exit")
        wx.EVT_MENU(self, 101, self.OnExit)
        menuBar.Append(menu, "&File")
        self.SetMenuBar(menuBar)

        EVT_NEWSELECTION(self,self.OnNewSelection)
        
        self.panel = wx.Panel(self, -1)

        # widgets for 'working' state
        self.statelabel_font = wx.Font(14,wx.DEFAULT,wx.NORMAL,wx.BOLD)
        self.statelabel = wx.StaticText(self.panel, -1, "Starting Up", (-1,-1), (-1,-1), wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        self.statelabel.SetFont(self.statelabel_font)
        self.infolabel = wx.StaticText(self.panel, -1, " ", (-1,-1), (-1,-1), wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)

        self.working_gauge = wx.Gauge(self.panel,-1,100)
        EVT_GAUGEUPDATE(self,self.OnGaugeUpdate)

        working_sizer = wx.BoxSizer(wx.VERTICAL)
        working_sizer.Add((-1,-1),1)
        working_sizer.Add(self.statelabel,0,wx.ALL | wx.ALIGN_CENTER,15)
        working_sizer.Add(self.working_gauge,0,wx.LEFT | wx.RIGHT | wx.EXPAND,30)
        working_sizer.Add(self.infolabel,0,wx.LEFT | wx.RIGHT | wx.EXPAND,30)
        working_sizer.Add((-1,-1),2)        

        # widgets for the 'ready' state
        self.tbfont = wx.Font(12,wx.DEFAULT,wx.NORMAL,wx.LIGHT,faceName=_settings.TEXT_PREVIEW_FONT_NAME)
        self.tbfontb = wx.Font(12,wx.DEFAULT,wx.NORMAL,wx.BOLD,faceName=_settings.TEXT_PREVIEW_FONT_NAME)
        self.tbstyle = wx.TextAttr(wx.BLACK, wx.NullColour, self.tbfont)
        self.tb = TextCtrlWithSelectionPopup(self.panel,self,-1,'',wx.DefaultPosition,(-1,-1),wx.TE_MULTILINE)
        self.tb.SetDefaultStyle(self.tbstyle)
        self.tb.SetEditable(False)
 
        self.url_label = wx.StaticText(self.panel,-1,"URL")
        self.fn_label = wx.StaticText(self.panel,-1,"Current")
        self.mtype_label = wx.StaticText(self.panel,-1,"Type")

        self.path_label = wx.StaticText(self.panel,-1,"Path")
        self.filename_label = wx.StaticText(self.panel,-1,"Filename")

        self.url_static = wx.StaticText(self.panel,-1,'')
        self.url_static.SetFont(self.tbfontb)
        self.fn_static = wx.StaticText(self.panel,-1,'')
        self.fn_static.SetFont(self.tbfontb)
        self.mtype_static = wx.StaticText(self.panel,-1,'')
        self.mtype_static.SetFont(self.tbfontb)

        self.path_input = wx.TextCtrl(self.panel,-1,'')
        self.path_input.SetDefaultStyle(self.tbstyle)
        self.filename_input = wx.TextCtrl(self.panel,-1,'')
        self.filename_input.SetDefaultStyle(self.tbstyle)

        self.label_align = wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL
        self.input_align = wx.EXPAND
        self.static_align = wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL
        self.ready_input_sizer = wx.FlexGridSizer(cols=2,hgap=6,vgap=6)
        if (wx.VERSION[1] >= 5):
            self.ready_input_sizer.SetFlexibleDirection(wx.HORIZONTAL)
        self.ready_input_sizer.AddGrowableCol(1)
        self.ready_input_sizer.AddMany([(self.url_label,0,self.label_align), (self.url_static,1,self.static_align),
                                        (self.fn_label,0,self.label_align), (self.fn_static,1,self.static_align),
                                        (self.mtype_label,0,self.label_align), (self.mtype_static,1,self.static_align),
                                        (self.path_label,0,self.label_align), (self.path_input,1,self.input_align),
                                   (self.filename_label,0,self.label_align), (self.filename_input,1,self.input_align)])

        self.viewcheck = wx.CheckBox(self.panel,-1,'Open document on exit');
        self.viewcheck.SetValue(_settings.VIEW_ON_EXIT)

        self.btn_go = wx.Button(self.panel,wx.ID_APPLY,'Apply')
        self.btn_view = wx.Button(self.panel,wx.ID_OPEN,'Open')
        self.btn_cancel = wx.Button(self.panel,wx.ID_EXIT,'Exit')

        wx.EVT_BUTTON(self,self.btn_go.GetId(),self.OnGo)
        wx.EVT_BUTTON(self,self.btn_cancel.GetId(),self.OnCancel)
        wx.EVT_BUTTON(self,self.btn_view.GetId(),self.OnView)        

        self.ready_btn_sizer = wx.GridSizer(cols=3)
        self.btn_align = wx.EXPAND | wx.LEFT | wx.RIGHT
        self.btn_space = 6
        self.ready_btn_sizer.Add(self.btn_go,1,self.btn_align,self.btn_space)
        self.ready_btn_sizer.Add(self.btn_view,1,self.btn_align,self.btn_space)
        self.ready_btn_sizer.Add(self.btn_cancel,1,self.btn_align,self.btn_space)

        ready_sizer = wx.BoxSizer(wx.VERTICAL)
        ready_sizer.Add(self.ready_input_sizer,0,wx.ALL | wx.EXPAND,10)
        ready_sizer.Add(self.viewcheck,0,wx.ALIGN_CENTER,0)
        ready_sizer.Add(self.ready_btn_sizer,0,wx.ALL | wx.EXPAND,5)
        ready_sizer.Add(self.tb,1,wx.ALL | wx.EXPAND,10)
        
        self.sizers = { 'working' : working_sizer, 'ready' : ready_sizer }
        self.mainsizer = wx.BoxSizer(wx.VERTICAL)
        w,h = self.panel.GetClientSizeTuple()
        self.mainsizer.SetDimension(0,0,w,h)
        for k in self.sizers:
            self.logger.info('adding sizer '+k)
            self.mainsizer.Add(self.sizers[k],1,wx.ALL | wx.EXPAND, 0)

        self.logger.info('Setting panel sizer')
        self.panel.SetSizer(self.mainsizer)

        self.state = 'working'
        self.ReLayout()
        self.logger.debug('panel layout')

        EVT_STATECHG(self,self.OnStateChange)
        EVT_FILEINFO(self,self.OnFileInfo)

        if url:
            self.AnalyzeFile(url)

    def AnalyzeFile(self,url):
        self.t = AnalyzerThread(self,url)
        self.t.start()

    def ReLayout(self):
        self.logger.debug('ReLayout')
        for k in self.sizers:
            if self.state == k:
                self.mainsizer.Show(self.sizers[k],True)
            else:
                self.mainsizer.Show(self.sizers[k],False)
        self.panel.Layout()
        
    def OnExit(self, evt):
        self.logger.debug('OnExit')
        self.status = 'exit'
        self.Close()

    def OnGaugeUpdate(self, evt):
        if self.state != 'working':
            return
        if evt.newrange != self.working_gauge.GetRange():
            self.working_gauge.SetRange(evt.newrange)
        self.working_gauge.SetValue(evt.newvalue)
        # This debugging statement produces copious output
        #        self.logger.debug('OnGaugeUpdate')

    def SetStateLabel(self,label):
        self.statelabel.SetLabel(label)
        self.panel.Layout()
        
    def OnStateChange(self, evt = None):
        self.logger.debug('OnStateChange(%s)',evt.newstate)
        if evt.newstate == 'download':
            if self.url:
                self.infolabel.SetLabel(self.url)
            self.SetStateLabel('Downloading')
            self.cur = wx.BusyCursor()
        elif evt.newstate == 'process':
            self.SetStateLabel('Processing')
            self.cur = wx.BusyCursor()
        else:
            self.SetStateLabel('Ready')
            self.cur = None

    def OnFileInfo(self, evt):
        self.logger.debug('OnFileInfo')
        self.file_data = evt.data
        self.opener = FileOpener(self.file_data['filename'],self.file_data['mtype'])
        if not self.opener.IsKnown():
            self.btn_view.Disable()
        if self.file_data['text']:
            self.tb.AppendText(wxscrub(string.join(self.file_data['text'],'\n')))
        else:
            self.tb.AppendText('(no text extracted from file)')
        self.tb.ShowPosition(0)
        self.url_static.SetLabel(self.file_data['url'])
        self.path_input.AppendText(self.base_path)
        if self.file_data['local']:
            self.fn_static.SetLabel(self.file_data['filename'])
            self.filename_input.AppendText(self.file_data['filename'])
            
            if self.file_data['dirname'].startswith(self.base_path):
                # We are already in a subdir of the preprint archive path,
                # so use the current dir as the default
                self.path_input.SetValue(self.file_data['dirname']+'/')
        else:
            self.fn_static.SetLabel('(a temporary file)')
        if self.file_data['mtype']:
            self.mtype_static.SetLabel(self.file_data['mtype'])
        else:
            self.mtype_static.SetLabel('(unknown)')
        self.state = 'ready'
        for i in [self.url_static,self.fn_static,self.mtype_static]:
            self.ready_input_sizer.SetItemMinSize(i,i.GetBestSize())
        self.ReLayout()
        self.Update()

    def OnNewSelection(self, evt):
        self.logger.debug('OnNewSelection')
        self.selection_data = SelectionData(evt.GetSelection(),extension='.'+self.file_data['rtype'],basedir=self.base_path)
        if self.selection_data['oneline']:
            menu = wx.Menu()
            menu.Append(REPLACE_PATH_ID,'Path "%(path)s"' % self.selection_data)
            menu.Append(REPLACE_FILENAME1_ID,'Filename "%(fn1)s"' % self.selection_data)
            wx.EVT_MENU(menu,REPLACE_PATH_ID,self.PopupMenuSelection)
            wx.EVT_MENU(menu,REPLACE_FILENAME1_ID,self.PopupMenuSelection)
            if self.selection_data['fn1'] != self.selection_data['fn2']:
                menu.Append(REPLACE_FILENAME2_ID,'Filename "%(fn2)s"' % self.selection_data)
            wx.EVT_MENU(menu,REPLACE_FILENAME2_ID,self.PopupMenuSelection)
            self.PopupMenu(menu,self.tb.GetPosition() + evt.GetPosition())

    def PopupMenuSelection(self,evt):
        if evt.GetId() == REPLACE_PATH_ID:
            self.path_input.SetValue(self.selection_data['path'])
        if evt.GetId() == REPLACE_FILENAME1_ID:
            self.filename_input.SetValue(self.selection_data['fn1'])
        if evt.GetId() == REPLACE_FILENAME2_ID:
            self.filename_input.SetValue(self.selection_data['fn2'])

    def OnGo(self,evt):
        path = self.path_input.GetValue()
        if not os.path.isdir(path):
            os.makedirs(path)
        if path[-1] != '/':
            path += '/'
        shutil.move(self.file_data['filename'],path + self.filename_input.GetValue())
        if self.viewcheck.IsChecked():
            self.opener = FileOpener(path + self.filename_input.GetValue(),self.file_data['mtype'])
            if self.opener.IsKnown():
                self.opener.Launch()
        self.OnExit(evt)

    def OnCancel(self,evt):
        self.OnExit(evt)
        
    def OnView(self,evt):
        if self.opener.IsKnown():
            self.opener.Launch()

if __name__ == '__main__':
    logging.basicConfig()
    logger = logging.getLogger('main')
    logger.setLevel(logging.INFO)

    # Test unicode availability
    if _settings.USE_UNICODE:
        if not wxHasUnicode():
            logger.warn('wxPython built without unicode support')
            _settings.USE_UNICODE = False

    #---- GUI START ----
    app = wx.PySimpleApp()
    frame = MyFrame(None, "dfiler %s" % _DFILER_VERSION, sys.argv[1])
    frame.Show()
    app.MainLoop()
